"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BullDataSource = exports.ErrorEnum = void 0;
const apollo_datasource_1 = require("apollo-datasource");
const json_1 = require("../../../services/json");
const gql_1 = require("../../../typings/gql");
const redis_info_1 = __importDefault(require("redis-info"));
const data_text_search_1 = require("./data-text-search");
const isNil_1 = __importDefault(require("lodash/isNil"));
const errors_1 = require("../errors");
var ErrorEnum;
(function (ErrorEnum) {
    ErrorEnum["QUEUE_NOT_FOUND"] = "Queue not found";
    ErrorEnum["JOB_NOT_FOUND"] = "Job not found";
    ErrorEnum["DATA_SEARCH_STATUS_REQUIRED"] = "Job status is required for data search";
    ErrorEnum["BAD_OFFSET"] = "Offset should be >= 0";
    ErrorEnum["BAD_LIMIT"] = "Limit should be >= 1";
})(ErrorEnum = exports.ErrorEnum || (exports.ErrorEnum = {}));
class BullDataSource extends apollo_datasource_1.DataSource {
    constructor(_queues, _config) {
        super();
        this._queues = _queues;
        this._config = _config;
        this._queuesMap = new Map();
        this._convertQueuesToMap(_queues);
    }
    _convertQueuesToMap(queues) {
        queues.forEach(queue => {
            this._queuesMap.set(queue.name, queue);
        });
    }
    _throwInternalError(e) {
        throw new errors_1.BullMonitorError(e);
    }
    _throwQueueNotFound() {
        this._throwInternalError(ErrorEnum.QUEUE_NOT_FOUND);
    }
    _throwJobNotFound() {
        this._throwInternalError(ErrorEnum.JOB_NOT_FOUND);
    }
    // queries
    getQueueByName(name, throwIfNotFound) {
        const queue = this._queuesMap.get(name);
        if (!queue && throwIfNotFound) {
            this._throwQueueNotFound();
        }
        return queue;
    }
    getQueues() {
        return this._queues;
    }
    getQueueJobs({ queue, limit = 20, offset = 0, status, id, ids, order = gql_1.OrderEnum.Desc, dataSearch, }) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!isNil_1.default(offset) && offset < 0) {
                this._throwInternalError(ErrorEnum.BAD_OFFSET);
            }
            if (!isNil_1.default(limit) && limit < 1) {
                this._throwInternalError(ErrorEnum.BAD_LIMIT);
            }
            const bullQueue = this.getQueueByName(queue, true);
            if (ids) {
                return yield Promise.all(ids.map(id => bullQueue.getJob(id))).then(this._filterJobs);
            }
            else if (id) {
                const job = yield bullQueue.getJob(id);
                return job ? [job] : [];
            }
            else if (dataSearch) {
                if (status) {
                    const searcher = new data_text_search_1.DataTextSearcher(bullQueue);
                    return yield searcher
                        .search({
                        status,
                        term: dataSearch.term,
                        key: dataSearch.key,
                        offset: offset,
                        limit: limit,
                        scanCount: this._config.textSearchScanCount,
                    })
                        .then(this._filterJobs);
                }
                else {
                    this._throwInternalError(ErrorEnum.DATA_SEARCH_STATUS_REQUIRED);
                }
            }
            else if (status) {
                return yield bullQueue
                    .getJobs([status], offset, offset + limit - 1, order === gql_1.OrderEnum.Asc)
                    .then(this._filterJobs);
            }
            {
                return [];
            }
        });
    }
    _filterJobs(jobs) {
        return jobs.filter(Boolean);
    }
    getJob(queueName, id, throwIfNotFound) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(queueName, true);
            const job = yield (queue === null || queue === void 0 ? void 0 : queue.getJob(id));
            if (!job && throwIfNotFound) {
                this._throwJobNotFound();
            }
            return job;
        });
    }
    getJobLogs(queueName, id) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(queueName, true);
            return yield (queue === null || queue === void 0 ? void 0 : queue.getJobLogs(id));
        });
    }
    getQueueJobsCounts(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(name);
            return yield (queue === null || queue === void 0 ? void 0 : queue.getJobCounts());
        });
    }
    getQueueFailedCount(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(name);
            return yield (queue === null || queue === void 0 ? void 0 : queue.getFailedCount());
        });
    }
    getQueueCompletedCount(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(name);
            return yield (queue === null || queue === void 0 ? void 0 : queue.getCompletedCount());
        });
    }
    getQueueDelayedCount(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(name);
            return yield (queue === null || queue === void 0 ? void 0 : queue.getDelayedCount());
        });
    }
    getQueueActiveCount(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(name);
            return yield (queue === null || queue === void 0 ? void 0 : queue.getActiveCount());
        });
    }
    getQueueWaitingCount(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(name);
            return yield (queue === null || queue === void 0 ? void 0 : queue.getWaitingCount());
        });
    }
    getQueuePausedCount(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(name);
            return yield (queue === null || queue === void 0 ? void 0 : queue.getPausedCount());
        });
    }
    getQueueWaitingOrDelayedJobsCount(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(name);
            return yield (queue === null || queue === void 0 ? void 0 : queue.count());
        });
    }
    getRedisInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._queuesMap.size > 0) {
                const firstQueue = this._queues[0];
                const rawInfo = yield firstQueue.client.info();
                return redis_info_1.default.parse(rawInfo);
            }
            return null;
        });
    }
    // mutations
    createJob({ queue: queueName, name = null, data = {}, options = {}, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(queueName, true);
            return yield (queue === null || queue === void 0 ? void 0 : queue.add(name, json_1.JsonService.maybeParse(data), json_1.JsonService.maybeParse(options)));
        });
    }
    removeJobsByPattern(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(args.queue, true);
            yield (queue === null || queue === void 0 ? void 0 : queue.removeJobs(args.pattern));
            return true;
        });
    }
    pauseQueue(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(name, true);
            yield (queue === null || queue === void 0 ? void 0 : queue.pause());
            return queue;
        });
    }
    cleanQueue(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(args.queue, true);
            return yield (queue === null || queue === void 0 ? void 0 : queue.clean(args.grace, args.status, args.limit || undefined));
        });
    }
    emptyQueue(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(args.queue, true);
            yield (queue === null || queue === void 0 ? void 0 : queue.empty());
            return queue;
        });
    }
    closeQueue(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(args.queue, true);
            yield (queue === null || queue === void 0 ? void 0 : queue.close());
            return queue;
        });
    }
    resumeQueue(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(args.queue, true);
            yield (queue === null || queue === void 0 ? void 0 : queue.resume());
            return queue;
        });
    }
    promoteJob(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id, true);
            yield (job === null || job === void 0 ? void 0 : job.promote());
            return job;
        });
    }
    discardJob(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id, true);
            yield (job === null || job === void 0 ? void 0 : job.discard());
            return job;
        });
    }
    updateJobData(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id, true);
            yield (job === null || job === void 0 ? void 0 : job.update(json_1.JsonService.maybeParse(args.data)));
            return job;
        });
    }
    createJobLog(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id, true);
            yield (job === null || job === void 0 ? void 0 : job.log(args.row));
            return job;
        });
    }
    retryJob(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id, true);
            yield (job === null || job === void 0 ? void 0 : job.retry());
            return job;
        });
    }
    retryJobs(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const jobs = yield Promise.all(args.jobs.map(jobId => this.getJob(args.queue, jobId, true)));
            yield Promise.all(jobs.map(job => job === null || job === void 0 ? void 0 : job.retry()));
            return jobs;
        });
    }
    removeJobById(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id, true);
            yield (job === null || job === void 0 ? void 0 : job.remove());
            return job;
        });
    }
    removeJobs(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const jobs = yield Promise.all(args.jobs.map(jobId => this.getJob(args.queue, jobId, true)));
            yield Promise.all(jobs.map(job => job === null || job === void 0 ? void 0 : job.remove()));
            return jobs;
        });
    }
    moveJobToCompleted(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id, true);
            yield (job === null || job === void 0 ? void 0 : job.moveToCompleted());
            return job;
        });
    }
    moveJobToFailed(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id, true);
            yield (job === null || job === void 0 ? void 0 : job.moveToFailed({
                message: '',
            }));
            return job;
        });
    }
}
exports.BullDataSource = BullDataSource;
//# sourceMappingURL=index.js.map